#ifndef _NICIP_H
#define _NICIP_H

#define DOS_DEVICE_PREFIX   TEXT("\\\\.\\")
#define DOS_TCP_NAME        TEXT("NICIP")
#define WIN32_TCP_NAME      TEXT("\\Device\\Tcp")

#define FSCTL_TCP_BASE      FILE_DEVICE_NETWORK

#define _TCP_CTL_CODE(function, method, access) \
            CTL_CODE(FSCTL_TCP_BASE, function, method, access)

#define IOCTL_TCP_QUERY_INFORMATION_EX  \
            _TCP_CTL_CODE(0, METHOD_NEITHER, FILE_ANY_ACCESS)

#define DISPLAY_IP_ADDRESS(IP)   printf("%d.%d.%d.%d", \
                                    (UCHAR)IP[0],      \
                                    (UCHAR)IP[1],      \
                                    (UCHAR)IP[2],      \
                                    (UCHAR)IP[3] )

#define DISPLAY_NIC_ADDRESS(NIC) \
                printf("%02X-%02X-%02X-%02X-%02X-%02X", \
                                    (UCHAR)(NIC[0]),    \
                                    (UCHAR)(NIC[1]),    \
                                    (UCHAR)(NIC[2]),    \
                                    (UCHAR)(NIC[3]),    \
                                    (UCHAR)(NIC[4]),    \
                                    (UCHAR)(NIC[5]))

typedef struct TDIEntityID { /* Structure of an entity ID. */
    ULONG        tei_entity;
    ULONG        tei_instance;
} TDIEntityID;


typedef struct TDIObjectID { /* Structure of an object ID.*/
    TDIEntityID    toi_entity;
    ULONG        toi_class;
    ULONG        toi_type;
    ULONG        toi_id;
} TDIObjectID;

#define    MAX_TDI_ENTITIES               512

#define    INFO_CLASS_GENERIC             0x100
#define    INFO_CLASS_PROTOCOL            0x200
#define    INFO_CLASS_IMPLEMENTATION      0x300

#define    INFO_TYPE_PROVIDER             0x100
#define    INFO_TYPE_ADDRESS_OBJECT       0x200
#define    INFO_TYPE_CONNECTION           0x300

#define    ENTITY_LIST_ID                 0
#define    GENERIC_ENTITY                 0

#define    CL_NL_ENTITY                   0x301
#define    IF_ENTITY                      0x200

#define    CONTEXT_SIZE                   16

/*
 * The ID to get the entity  type. 
 * The return from this type is an unsigned integer 
 */
#define    ENTITY_TYPE_ID                1

/*  Valid values to get back from entity type ID query. */

#define    CO_TL_TCP       0x404   /* Entity implements TCP prot.*/

#define    CL_NL_IP        0x303   /* Entity implements IP.      */

#define    IF_GENERIC      0x200   /*  Generic interface.        */
#define    IF_MIB          0x202   /* Supports MIB-2 interface.  */

/*
 *   QueryInformationEx IOCTL. The return buffer is passed as the 
 *   OutputBufferin the DeviceIoControl request. This structure is
 *   passed as the InputBuffer.
 */
struct tcp_request_query_information_ex {

    TDIObjectID   ID;                   /* object ID to query.  */
    UCHAR         Context[CONTEXT_SIZE];/* multi-request context*/ 

};

typedef struct tcp_request_query_information_ex
                TCP_REQUEST_QUERY_INFORMATION_EX,
                *PTCP_REQUEST_QUERY_INFORMATION_EX;

/*
 * SetInformationEx IOCTL request structure. This structure is 
 * passed as the InputBuffer. The space allocated for the structure
 * must be large enough to contain the structure and the set data 
 * buffer, which begins at the Buffer field. The OutputBuffer 
 * parameter in the DeviceIoControl is not used.
 */
struct tcp_request_set_information_ex {
    TDIObjectID   ID;          /* object ID to set.            */
    unsigned int  BufferSize;  /* size of set data buffer      */
    unsigned char Buffer[1];   /* beginning of set data buffer */
};

typedef struct IPSNMPInfo {
    ULONG        ipsi_forwarding;
    ULONG        ipsi_defaultttl;
    ULONG        ipsi_inreceives;
    ULONG        ipsi_inhdrerrors;
    ULONG        ipsi_inaddrerrors;
    ULONG        ipsi_forwdatagrams;
    ULONG        ipsi_inunknownprotos;
    ULONG        ipsi_indiscards;
    ULONG        ipsi_indelivers;                                                
    ULONG        ipsi_outrequests;
    ULONG        ipsi_routingdiscards;
    ULONG        ipsi_outdiscards;
    ULONG        ipsi_outnoroutes;
    ULONG        ipsi_reasmtimeout;
    ULONG        ipsi_reasmreqds;
    ULONG        ipsi_reasmoks;
    ULONG        ipsi_reasmfails;
    ULONG        ipsi_fragoks;
    ULONG        ipsi_fragfails;
    ULONG        ipsi_fragcreates;
    ULONG        ipsi_numif;
    ULONG        ipsi_numaddr;
    ULONG        ipsi_numroutes;
} IPSNMPInfo;

typedef struct IPAddrEntry {
    ULONG        iae_addr;
    ULONG        iae_index;
    ULONG        iae_mask;
    ULONG        iae_bcastaddr;
    ULONG        iae_reasmsize;
    USHORT       iae_context;
    USHORT       iae_pad;
} IPAddrEntry;

#define    IP_MIB_STATS_ID                  1
#define    IP_MIB_ADDRTABLE_ENTRY_ID        0x102


#define    IF_MIB_STATS_ID                  1
#define    MAX_PHYSADDR_SIZE                8
#define    MAX_IFDESCR_LEN                  256

typedef struct IFEntry {
    ULONG            if_index;
    ULONG            if_type;
    ULONG            if_mtu;
    ULONG            if_speed;
    ULONG            if_physaddrlen;
    UCHAR            if_physaddr[MAX_PHYSADDR_SIZE];
    ULONG            if_adminstatus;
    ULONG            if_operstatus;
    ULONG            if_lastchange;
    ULONG            if_inoctets;
    ULONG            if_inucastpkts;
    ULONG            if_innucastpkts;
    ULONG            if_indiscards;
    ULONG            if_inerrors;
    ULONG            if_inunknownprotos;
    ULONG            if_outoctets;
    ULONG            if_outucastpkts;
    ULONG            if_outnucastpkts;
    ULONG            if_outdiscards;
    ULONG            if_outerrors;
    ULONG            if_outqlen;
    ULONG            if_descrlen;
    UCHAR            if_descr[1];
} IFEntry;


#define INIT_TCP_REQUEST(TcpReqStruct, instance, entity, \
                                        class, type, id) \
        do{    \
            ZeroMemory(&TcpReqStruct, sizeof(TcpReqStruct));       \
            TcpReqStruct.ID.toi_entity.tei_instance  = instance;   \
            TcpReqStruct.ID.toi_entity.tei_entity    = entity;     \
            TcpReqStruct.ID.toi_class                = class;      \
            TcpReqStruct.ID.toi_type                 = type;       \
            TcpReqStruct.ID.toi_id                   = id;         \
        }while(0);

#endif /* _NICIP_H */