/************************ Module Header ************************
* Module Name: SIMPROF.H
***************************************************************/

// How to use:
// Instantiates a SimpleProfiler object and at block exit
// dtor shows time passed between creation and destruction

// If you are interested in fine milliseconds count 
// DON'T use this code

#ifndef __SIMPROF__

#define __SIMPROF__

#include <windows.h>
#include <stdio.h>
#include <string.h>

class SimpleProfiler
{
public:
    // if it's passed a valid window then time string
    // is displayed on window caption otherwise on message box
    SimpleProfiler(char* pszInfo = 0, 
                   HWND hWnd = 0) : m_pszInfo(0), m_hMessageWnd(0)
    {
        char* p = pszInfo ? pszInfo : "Time elapsed";

        m_pszInfo = new char[strlen(p) + 1];

        if (m_pszInfo)
            strcpy(m_pszInfo, p);

        if (hWnd && ::IsWindow(hWnd))
            m_hMessageWnd = hWnd;
        m_dwStartTick = ::GetTickCount();
    };
    
    ~SimpleProfiler()
    {
        DWORD dwFinal = ::GetTickCount() - m_dwStartTick;
        char* p;

        // 20 is a reasonable size for number
        // space for null terminating char is added by sizeof("..")
        p = new char[(m_pszInfo ? strlen(m_pszInfo) : 0) + 
                     20 + sizeof(":  ms")]; 

        if (p)
        {
            ::sprintf(p, "%s: %ld ms", 
                      m_pszInfo ? m_pszInfo : "", dwFinal);

            if (m_hMessageWnd)
                ::SetWindowText(m_hMessageWnd, p);
            else
                ::MessageBox(NULL, p, "Simple Profiler",
                             MB_ICONINFORMATION | MB_OK);
            delete [] p;
        }
        else
            ::MessageBox(NULL, "Memory error", "Simple Profiler",
                         MB_ICONINFORMATION | MB_OK);

        delete [] m_pszInfo;
    };
protected:
    DWORD m_dwStartTick;
    HWND m_hMessageWnd;
    char* m_pszInfo;
};

#endif //__SIMPROF__
