#include <windows.h>
#include <process.h>
#include <limits.h>
#include <stdio.h>
#include "thread.h"

//---------------------------------------------------------------
typedef struct tagProcdata {
    tagProcdata(void *Arg,unsigned int (__stdcall * InitFunc) \
        (void *)) : pArg(Arg), pInitFunc(InitFunc)
    {
    }

    void *pArg;
    unsigned int (__stdcall * pInitFunc) (void *);
} Procdata;

//---------------------------------------------------------------
struct {
    DWORD   dwExceptionCode;
    char    *szMessage;
} msgTable[] = {
   { STATUS_SEGMENT_NOTIFICATION,     NULL },
   { STATUS_BREAKPOINT,               NULL },
   { STATUS_SINGLE_STEP,              NULL },
   { STATUS_WAIT_0,                   "Wait 0" },
   { STATUS_ABANDONED_WAIT_0,         "Abandoned Wait 0" },
   { STATUS_USER_APC,                 "User APC" },
   { STATUS_TIMEOUT,                  "Timeout" },
   { STATUS_PENDING,                  "Pending" },
   { STATUS_GUARD_PAGE_VIOLATION,     "Guard Page Violation" },
   { STATUS_DATATYPE_MISALIGNMENT,    "Data Type Misalignment" },
   { STATUS_ACCESS_VIOLATION,         "Access Violation" },
   { STATUS_IN_PAGE_ERROR,            "In Page Error" },
   { STATUS_NO_MEMORY,                "No Memory" },
   { STATUS_ILLEGAL_INSTRUCTION,      "Illegal Instruction" },
   { STATUS_NONCONTINUABLE_EXCEPTION, "Noncontinuable Exception" },
   { STATUS_INVALID_DISPOSITION,      "Invalid Disposition" },
   { STATUS_ARRAY_BOUNDS_EXCEEDED,    "Array Bounds Exceeded" },
   { STATUS_FLOAT_DENORMAL_OPERAND,   "Float Denormal Operand" },
   { STATUS_FLOAT_DIVIDE_BY_ZERO,     "Divide by Zero" },
   { STATUS_FLOAT_INEXACT_RESULT,     "Float Inexact Result" },
   { STATUS_FLOAT_INVALID_OPERATION,  "Float Invalid Operation" },
   { STATUS_FLOAT_OVERFLOW,           "Float Overflow" },
   { STATUS_FLOAT_STACK_CHECK,        "Float Stack Check" },
   { STATUS_FLOAT_UNDERFLOW,          "Float Uderflow" },
   { STATUS_INTEGER_DIVIDE_BY_ZERO,   "Integer Divide by Zero" },
   { STATUS_INTEGER_OVERFLOW,         "Integer Overflow" },
   { STATUS_PRIVILEGED_INSTRUCTION,   "Privileged Instruction" },
   { STATUS_STACK_OVERFLOW,           "Stack Overflow" },
   { STATUS_CONTROL_C_EXIT,           "Ctrl+C Exit" },
   { 0xFFFFFFFF,                      "" }
};

LONG DoHandleDebugEvent(LPEXCEPTION_POINTERS lpEP)
{
    PVOID exceptionAddr = lpEP->ExceptionRecord->ExceptionAddress;
    DWORD exceptionCode = lpEP->ExceptionRecord->ExceptionCode;
    char  *szName = NULL;
    int   i;

    for (i=0; msgTable[i].dwExceptionCode != 0xFFFFFFFF; ++i) {
        if (msgTable[i].dwExceptionCode == exceptionCode) {
            szName = msgTable[i].szMessage;
            if (szName == NULL)
                return DBG_EXCEPTION_NOT_HANDLED;
            break;
        }
    }

    if (szName == NULL)
        szName = "Unknown exception";

    char sMsg[UCHAR_MAX];
    sprintf(sMsg,"Exception: %s at address %08X in worker " \
           "thread %X",szName,exceptionAddr,GetCurrentThreadId());
    MessageBox(NULL,sMsg,"MyTestApplication", MB_TASKMODAL|MB_OK);
    return 0;
}

//---------------------------------------------------------------
typedef BOOL (WINAPI * ISDEBUGGERPRESENT) ();

//---------------------------------------------------------------
int InfoExceptionFilter(LPEXCEPTION_POINTERS lpExceptPtr)
{   
    DoHandleDebugEvent(lpExceptPtr);
    return EXCEPTION_EXECUTE_HANDLER;
}

//---------------------------------------------------------------
BOOL IsInDebugger()
{
    HANDLE hMod = GetModuleHandle("KERNEL32.DLL");
    if (hMod) {
        ISDEBUGGERPRESENT pProc = (ISDEBUGGERPRESENT) \
                        GetProcAddress(hMod,"IsDebuggerPresent");
        if (pProc)
            return pProc();
    }
    return FALSE;
}

//---------------------------------------------------------------
unsigned int __stdcall save_thread_proc(void * arg)
{
    Procdata *pInitDatat = (Procdata *) arg;
    unsigned int ret = 0;

    if (!IsInDebugger())  {
        __try
            {
            ret = pInitDatat->pInitFunc(pInitDatat->pArg);
            }
        __except(InfoExceptionFilter(GetExceptionInformation()))
            {
            }
    } else  
        ret = pInitDatat->pInitFunc(pInitDatat->pArg);
  
    delete pInitDatat;
    return ret;
}

//---------------------------------------------------------------
unsigned long sys_beginthreadex(void *security, unsigned int \
    stacksize,unsigned int  (__stdcall * initialcode) (void *),
    void *argument,unsigned int createflag,unsigned int *thrdaddr)
{
    Procdata *pNew = new Procdata(argument,initialcode);
    return _beginthreadex(security,stacksize,&save_thread_proc, \
                            (void *)pNew,createflag,thrdaddr);
}
// End of File
